@@ -1,8 +1,8 @@
-__version__ = "0.3.2"
+__version__ = "0.3.3"
 
 from .symbol import Symbol, register_type
 from .expression import Expr, Head, symbol
-from .macro import Macro, MacroFlags, MacroFlagOptions
+from .macro import Macro, MacroFlags
 from .ast import parse
 from functools import wraps
 
@@ -38,11 +38,12 @@ def get_macro(mapping=None) -> str:
         macro = _MACRO
     return str(macro)
 
-def set_flags(**flags: MacroFlagOptions):
+def set_flags(Get: bool = True, Set: bool = True, Delete: bool = True, 
+              Return: bool = True):
     """
     Set macro flags.
     """    
-    _MACRO._flags = MacroFlags(**flags)
+    _MACRO._flags = MacroFlags(Get, Set, Delete, Return)
     return None
 
 del wraps
\ No newline at end of file
