@@ -23,7 +23,7 @@ class Validator(Generic[_T, _A]):
         try:
             func = self._map[arg]
         except KeyError:
-            return arg
+            return args
         try:
             out = func(*args)
         except ValidationError as e:
@@ -43,6 +43,7 @@ def _no_arg(args):
     return args
 
 @validator.register(Head.del_)
+@validator.register(Head.raise_)
 def _single_arg(args):
     if len(args) != 1:
         raise ValidationError()
