@@ -11,39 +11,39 @@ from .expression import Expr, Head, symbol
 NoneType = type(None)
 
 AST_BINOP_MAP = {
-    ast.Add: Symbol("+"),
-    ast.Sub: Symbol("-"),
-    ast.Mult: Symbol("*"),
-    ast.Div: Symbol("/"),
-    ast.Mod: Symbol("%"),
-    ast.Eq: Symbol("=="),
-    ast.NotEq: Symbol("!="),
-    ast.Gt: Symbol(">"),
-    ast.GtE: Symbol(">="),
-    ast.Lt: Symbol("<"),
-    ast.LtE: Symbol("<="),
-    ast.Is: Symbol("is"),
-    ast.IsNot: Symbol("is not"),
-    ast.In: Symbol("in"),
-    ast.NotIn: Symbol("not in"),
-    ast.Pow: Symbol("**"),
-    ast.MatMult: Symbol("@"),
-    ast.FloorDiv: Symbol("//"),
-    ast.BitAnd: Symbol("&"),
-    ast.BitOr: Symbol("|"),
-    ast.BitXor: Symbol("^"),
-    ast.And: Symbol("and"),
-    ast.Or: Symbol("or"),
+    ast.Add: Symbol._reserved("+"),
+    ast.Sub: Symbol._reserved("-"),
+    ast.Mult: Symbol._reserved("*"),
+    ast.Div: Symbol._reserved("/"),
+    ast.Mod: Symbol._reserved("%"),
+    ast.Eq: Symbol._reserved("=="),
+    ast.NotEq: Symbol._reserved("!="),
+    ast.Gt: Symbol._reserved(">"),
+    ast.GtE: Symbol._reserved(">="),
+    ast.Lt: Symbol._reserved("<"),
+    ast.LtE: Symbol._reserved("<="),
+    ast.Is: Symbol._reserved("is"),
+    ast.IsNot: Symbol._reserved("is not"),
+    ast.In: Symbol._reserved("in"),
+    ast.NotIn: Symbol._reserved("not in"),
+    ast.Pow: Symbol._reserved("**"),
+    ast.MatMult: Symbol._reserved("@"),
+    ast.FloorDiv: Symbol._reserved("//"),
+    ast.BitAnd: Symbol._reserved("&"),
+    ast.BitOr: Symbol._reserved("|"),
+    ast.BitXor: Symbol._reserved("^"),
+    ast.And: Symbol._reserved("and"),
+    ast.Or: Symbol._reserved("or"),
 }
 
 AST_UNOP_MAP = {
-    ast.UAdd: Symbol("+"),
-    ast.USub: Symbol("-"),
-    ast.Not: Symbol("not "),
-    ast.Invert: Symbol("~"),
+    ast.UAdd: Symbol._reserved("+"),
+    ast.USub: Symbol._reserved("-"),
+    ast.Not: Symbol._reserved("not "),
+    ast.Invert: Symbol._reserved("~"),
 }
 
-def parse(source: str | Callable) -> Expr | Symbol:
+def parse(source: str | Callable, squeeze: bool = True) -> Expr | Symbol:
     """
     Convert Python code string into Expr/Symbol objects.
     """
@@ -51,11 +51,14 @@ def parse(source: str | Callable) -> Expr | Symbol:
         source = inspect.getsource(source)
     body = ast.parse(source).body
     if len(body) == 1:
-        ast_object = ast.parse(source).body[0]
+        ast_object = body[0]
     else:
-        ast_object = ast.parse(source).body
+        ast_object = body
     
-    return from_ast(ast_object)
+    out = from_ast(ast_object)
+    if not squeeze and len(body) == 1:
+        out = Expr(Head.block, [out])
+    return out
 
 @singledispatch
 def from_ast(ast_object: ast.AST | list | NoneType):
@@ -101,6 +104,9 @@ else:
 
 @from_ast.register
 def _(ast_object: ast.Name):
+    name = ast_object.id
+    if name in Symbol._module_map.keys():
+        return Symbol._module_map[name]
     return Symbol(ast_object.id)
 
 @from_ast.register
@@ -109,7 +115,7 @@ def _(ast_object: ast.Expr):
 
 @from_ast.register
 def _(ast_object: ast.UnaryOp):
-    return Expr(Head.unop, [AST_UNOP_MAP[ast_object.op], from_ast(ast_object.operand)])
+    return Expr(Head.unop, [AST_UNOP_MAP[type(ast_object.op)], from_ast(ast_object.operand)])
 
 @from_ast.register
 def _(ast_object: ast.AugAssign):
@@ -251,7 +257,22 @@ def _(ast_object: ast.AnnAssign):
 
 @from_ast.register
 def _(ast_object: ast.Pass):
-    return Symbol.var("pass") # Should it be Expr(Head.pass_, [Symbol("pass")]) ??
+    return Symbol.var("pass")
+
+@from_ast.register
+def _(ast_object: ast.Break):
+    return Symbol.var("break")
+
+@from_ast.register
+def _(ast_object: ast.Continue):
+    return Symbol.var("continue")
+
+@from_ast.register
+def _(ast_object: ast.Raise):
+    exc = ast_object.exc
+    if ast_object.cause:
+        raise ValueError("'raise XX from YY' is not supported yet")
+    return Expr(Head.raise_, [from_ast(exc)])
     
 @from_ast.register
 def _(ast_object: ast.Return):
