@@ -1,50 +1,55 @@
 from __future__ import annotations
 from copy import deepcopy
 from typing import Callable, Iterable, Iterator, Any, Hashable, overload
+from types import ModuleType
 from numbers import Number
 from .symbol import Symbol
 from .head import Head, EXEC
 from ._validator import validator
 
-def as_str(expr: Any, indent: int = 0):
+def str_(expr: Any, indent: int = 0):
     if isinstance(expr, Expr):
         return _STR_MAP[expr.head](expr, indent)
     else:
         return " "*indent + str(expr)
 
 def rm_par(s: str):
+    """
+    Remove parenthesis.
+    """
     if s[0] == "(" and s[-1] == ")":
         s = s[1:-1]
     return s
 
 def sjoin(sep: str, iterable: Iterable[Any], indent: int = 0):
-    return sep.join(as_str(expr, indent) for expr in iterable)
+    return sep.join(str_(expr, indent) for expr in iterable)
 
 _STR_MAP: dict[Head, Callable[[Expr, int], str]] = {
     Head.empty    : lambda e, i: "",
-    Head.getattr  : lambda e, i: f"{as_str(e.args[0], i)}.{as_str(e.args[1])}",
-    Head.getitem  : lambda e, i: f"{as_str(e.args[0], i)}[{as_str(e.args[1])}]",
-    Head.del_     : lambda e, i: " "*i + f"del {as_str(e.args[0])}",
-    Head.call     : lambda e, i: f"{as_str(e.args[0], i)}({sjoin(', ', e.args[1:])})",
-    Head.assign   : lambda e, i: f"{as_str(e.args[0], i)} = {e.args[1]}",
-    Head.kw       : lambda e, i: f"{as_str(e.args[0])}={as_str(e.args[1])}",
-    Head.assert_  : lambda e, i: " "*i + f"assert {as_str(e.args[0])}, {as_str(e.args[1])}".rstrip(", "),
+    Head.getattr  : lambda e, i: f"{str_(e.args[0], i)}.{str_(e.args[1])}",
+    Head.getitem  : lambda e, i: f"{str_(e.args[0], i)}[{str_(e.args[1])}]",
+    Head.del_     : lambda e, i: " "*i + f"del {str_(e.args[0])}",
+    Head.call     : lambda e, i: f"{str_(e.args[0], i)}({sjoin(', ', e.args[1:])})",
+    Head.assign   : lambda e, i: f"{str_(e.args[0], i)} = {e.args[1]}",
+    Head.kw       : lambda e, i: f"{str_(e.args[0])}={str_(e.args[1])}",
+    Head.assert_  : lambda e, i: " "*i + f"assert {str_(e.args[0])}, {str_(e.args[1])}".rstrip(", "),
     Head.comment  : lambda e, i: " "*i + f"# {e.args[0]}",
     
-    Head.unop     : lambda e, i: " "*i + f"({as_str(e.args[0])}{as_str(e.args[1])})",
-    Head.binop    : lambda e, i: " "*i + f"({as_str(e.args[1])} {as_str(e.args[0])} {as_str(e.args[2])})",
-    Head.aug      : lambda e, i: " "*i + f"{as_str(e.args[1])} {as_str(e.args[0])}= {as_str(e.args[2])}",
+    Head.unop     : lambda e, i: " "*i + f"({str_(e.args[0])}{str_(e.args[1])})",
+    Head.binop    : lambda e, i: " "*i + f"({str_(e.args[1])} {str_(e.args[0])} {str_(e.args[2])})",
+    Head.aug      : lambda e, i: " "*i + f"{str_(e.args[1])} {str_(e.args[0])}= {str_(e.args[2])}",
     
     Head.block    : lambda e, i: sjoin("\n", e.args, i),
-    Head.function : lambda e, i: " "*i + f"def {as_str(e.args[0])}:\n{as_str(e.args[1], i+4)}",
+    Head.function : lambda e, i: " "*i + f"def {str_(e.args[0])}:\n{str_(e.args[1], i+4)}",
     Head.return_  : lambda e, i: " "*i + f"return {sjoin(', ', e.args)}",
-    Head.if_      : lambda e, i: " "*i + f"if {rm_par(as_str(e.args[0]))}:\n{as_str(e.args[1], i+4)}\n" + \
-                                 " "*i + f"else:\n{as_str(e.args[2], i+4)}",
-    Head.elif_    : lambda e, i: " "*i + f"if {rm_par(as_str(e.args[0]))}:\n{as_str(e.args[1], i+4)}\n" + \
-                                 " "*i + f"else:\n{as_str(e.args[2], i+4)}",
-    Head.for_     : lambda e, i: " "*i + f"for {rm_par(as_str(e.args[0]))}:\n{as_str(e.args[1], i+4)}",
-    Head.while_   : lambda e, i: " "*i + f"while {rm_par(as_str(e.args[0]))}:\n{as_str(e.args[1], i+4)}",
-    Head.annotate : lambda e, i: f"{as_str(e.args[0], i)}: {as_str(e.args[1])}"
+    Head.raise_   : lambda e, i: " "*i + f"raise {str_(e.args[0])}",
+    Head.if_      : lambda e, i: " "*i + f"if {rm_par(str_(e.args[0]))}:\n{str_(e.args[1], i+4)}\n" + \
+                                 " "*i + f"else:\n{str_(e.args[2], i+4)}",
+    Head.elif_    : lambda e, i: " "*i + f"if {rm_par(str_(e.args[0]))}:\n{str_(e.args[1], i+4)}\n" + \
+                                 " "*i + f"else:\n{str_(e.args[2], i+4)}",
+    Head.for_     : lambda e, i: " "*i + f"for {rm_par(str_(e.args[0]))}:\n{str_(e.args[1], i+4)}",
+    Head.while_   : lambda e, i: " "*i + f"while {rm_par(str_(e.args[0]))}:\n{str_(e.args[1], i+4)}",
+    Head.annotate : lambda e, i: f"{str_(e.args[0], i)}: {str_(e.args[1])}"
 }
 
 class Expr:
@@ -73,10 +78,10 @@ class Expr:
         return f":({s})"
     
     def __str__(self) -> str:
-        return as_str(self)
+        return str_(self)
     
-    def __eq__(self, expr: Expr | Symbol) -> bool:
-        if isinstance(expr, self.__class__):
+    def __eq__(self, expr) -> bool:
+        if isinstance(expr, Expr):
             if self.head == expr.head:
                 return self.args == expr.args
             else:
@@ -91,7 +96,7 @@ class Expr:
         out = [f"head: {self.head.name}\n{' '*ind}args:\n"]
         for i, arg in enumerate(self.args):
             if isinstance(arg, Symbol):
-                value = arg
+                value = arg.name
             else:
                 value = arg._dump(ind+4)
             out.append(f"{i:>{ind+2}}: {value}\n")
@@ -108,12 +113,28 @@ class Expr:
         # Always copy object deeply.
         return deepcopy(self)
     
-    def eval(self, _globals: dict[Symbol|str, Any] = {}, _locals: dict = {}):
+    def at(self, *indices: int) -> Symbol | Expr:
+        """
+        Helper function to avoid ``expr.args[0].args[0] ...``. Also, exception descriptions
+        during this function call. ``expr.at(i, j, k)`` is equivalent to
+        ``expr.args[i].args[j].args[k]``.
+        """ 
+        now = self
+        for i in indices:
+            try:
+                now = now._args[i]
+            except IndexError as e:
+                raise type(e)(f"list index out of range at position {i}.")
+            except AttributeError as e:
+                raise type(e)(f"Indexing encounted Symbol at position {i}.")
+        return now
+    
+    def eval(self, _globals: dict[Symbol | str, Any] = {}, _locals: dict = {}):
         """
         Evaluate or execute macro as an Python script.
         
-        Either ``eval`` or ``exec`` is called for every expressions, as determined by its
-        header. Calling this function is much safer than those not-recommended usage of 
+        Either ``eval`` or ``exec`` will get called, which determined by its header.
+        Calling this function is much safer than those not-recommended usage of 
         ``eval`` or ``exec``.
         
         Parameters
@@ -124,22 +145,34 @@ class Expr:
             Updated variable namespace. Will be a mapping from symbols to values.
             
         """        
-        _globals = {(sym.name if isinstance(sym, Symbol) else sym): v 
-                    for sym, v in _globals.items()}
-        # TODO: Here should be some better ways to assign proper scope.
+        _glb: dict[str, Any] = {(sym.name if isinstance(sym, Symbol) else sym): v 
+                                for sym, v in _globals.items()}
+        
+        # use registered modules
+        if Symbol._module_symbols:
+            format_dict: dict[Symbol, Symbol] = {}
+            for id_, sym in Symbol._module_symbols.items():
+                mod = Symbol._module_map[sym.name]
+                vstr = f"var{hex(id_)}"
+                format_dict[sym] = Symbol(vstr)
+                _glb[vstr] = mod
+            # Modules will not be registered as alias ("np" will be "numpy" in macro).
+            # To avoid name collision, it is safer to rename them to "var0x...".
+            self = self.format(format_dict)
+        
         if self.head in EXEC:
-            return exec(str(self), _globals, _locals)
+            return exec(str(self), _glb, _locals)
         else:
-            return eval(str(self), _globals, _locals)
+            return eval(str(self), _glb, _locals)
         
     @classmethod
-    def parse_method(cls, obj: Any, func: Callable, args: tuple[Any, ...], kwargs: dict[str, Any]) -> Expr:
+    def parse_method(cls, obj: Any, func: Callable, args: tuple[Any, ...] = None, 
+                     kwargs: dict[str, Any] = None) -> Expr:
         """
         Parse ``obj.func(*args, **kwargs)``.
         """
         method = cls(head=Head.getattr, args=[symbol(obj), func])
-        inputs = [method] + cls.convert_args(args, kwargs)
-        return cls(head=Head.call, args=inputs)
+        return cls.parse_call(method, args, kwargs)
 
     @classmethod
     def parse_init(cls, 
@@ -152,18 +185,12 @@ class Expr:
         """
         if init_cls is None:
             init_cls = type(obj)
-        if args is None:
-            args = ()
-        if kwargs is None:
-            kwargs = {}
         sym = symbol(obj)
-        return cls(Head.assign, [sym, 
-                                 cls(Head.call, [init_cls] + cls.convert_args(args, kwargs))
-                                 ])
+        return cls(Head.assign, [sym, cls.parse_call(init_cls, args, kwargs)])
     
     @classmethod
     def parse_call(cls, 
-                   func: Callable, 
+                   func: Callable | Symbol | Expr, 
                    args: tuple[Any, ...] = None, 
                    kwargs: dict[str, Any] = None) -> Expr:
         """
@@ -171,8 +198,12 @@ class Expr:
         """
         if args is None:
             args = ()
+        elif not isinstance(args, tuple):
+            raise TypeError("args must be a tuple")
         if kwargs is None:
             kwargs = {}
+        elif not isinstance(kwargs, dict):
+            raise TypeError("kwargs must be a dict")
         inputs = [func] + cls.convert_args(args, kwargs)
         return cls(head=Head.call, args=inputs)
     
@@ -199,14 +230,20 @@ class Expr:
             inputs.append(a)
                 
         for k, v in kwargs.items():
-            inputs.append(cls(Head.kw, [Symbol(k), v]))
+            inputs.append(cls(Head.kw, [Symbol(k), symbol(v)]))
         return inputs
     
     @classmethod
     def parse_object(cls, a: Any) -> Symbol | Expr:
+        """
+        Convert an object into a macro-type.
+        """
         return a if isinstance(a, cls) else symbol(a)
     
     def issetattr(self) -> bool:
+        """
+        Determine if an expression is in the form of ``setattr(obj, key value)``.
+        """
         if self.head == Head.assign:
             target = self.args[0]
             if isinstance(target, Expr) and target.head == Head.getattr:
@@ -214,6 +251,9 @@ class Expr:
         return False
     
     def issetitem(self) -> bool:
+        """
+        Determine if an expression is in the form of ``setitem(obj, key value)``.
+        """
         if self.head == Head.assign:
             target = self.args[0]
             if isinstance(target, Expr) and target.head == Head.getitem:
@@ -225,12 +265,12 @@ class Expr:
         Recursively iterate along all the arguments.
         """
         for arg in self.args:
-            if isinstance(arg, self.__class__):
+            if isinstance(arg, Expr):
                 yield from arg.iter_args()
             elif isinstance(arg, Symbol):
                 yield arg
             else:
-                raise RuntimeError(arg)
+                raise RuntimeError(f"{arg} (type {type(arg)})")
         
     def iter_expr(self) -> Iterator[Expr]:
         """
@@ -252,7 +292,7 @@ class Expr:
     @overload
     def format(self, mapping: Iterable[tuple[Any, Symbol|Expr]], inplace: bool = False) -> Expr:...
     
-    def format(self, mapping: dict[Symbol, Symbol|Expr], inplace: bool = False) -> Expr:
+    def format(self, mapping, inplace=False) -> Expr:
         """
         Format expressions in the macro.
         
@@ -296,7 +336,7 @@ class Expr:
                     self.args[i] = new
         return self
 
-def check_format_mapping(mapping_list: Iterable[tuple[Any, Any]]) -> dict[Symbol, Symbol|Expr]:
+def check_format_mapping(mapping_list: Iterable) -> dict[Symbol, Symbol|Expr]:
     _dict: dict[Symbol, Symbol] = {}
     for comp in mapping_list:
         if len(comp) != 2:
@@ -305,10 +345,15 @@ def check_format_mapping(mapping_list: Iterable[tuple[Any, Any]]) -> dict[Symbol
         if isinstance(v, Expr) and v.head in EXEC:
             raise ValueError("Cannot replace a symbol to a non-evaluable expression.")
         
+        key = symbol(k)
+        if isinstance(key, Expr):
+            raise TypeError(f"Object of type {type(k).__name__} returns Expr type, thus cannot"
+                             "be used as a format template.")
         if isinstance(v, str) and not isinstance(k, Symbol):
-            _dict[symbol(k)] = Symbol(v)
+            value = Symbol(v)
         else:
-            _dict[symbol(k)] = symbol(v)
+            value = symbol(v)
+        _dict[key] = value
     return _dict
 
 
@@ -370,6 +415,22 @@ def symbol(obj: Any, constant: bool = True) -> Symbol | Expr:
             seq = f"{obj_type.__name__}({seq})"
     elif isinstance(obj, Number): # int, float, bool, ...
         seq = obj
+    elif isinstance(obj, ModuleType):
+        # Register module to the default namespace of Symbol class. This function is
+        # called every time a module type object is converted to a Symbol because users
+        # always have to pass the module object to the global variables when calling 
+        # eval function.
+        if obj_id in Symbol._module_symbols.keys():
+            sym = Symbol._module_symbols[obj_id]
+        else:
+            *main, seq = obj.__name__.split(".")
+            sym = Symbol(seq, obj_id)
+            sym.constant = True
+            if len(main) == 0:
+                # submodules should not be registered
+                Symbol._module_symbols[obj_id] = sym
+                Symbol._module_map[seq] = obj
+        return sym
     elif hasattr(obj, "__name__"):
         seq = obj.__name__
     else:
