@@ -19,29 +19,29 @@ _INHERITABLE = ("__module__", "__name__", "__qualname__", "__doc__", "__annotati
 
 
 BINOP_MAP = {
-    "__add__": Symbol("+"),
-    "__sub__": Symbol("-"),
-    "__mul__": Symbol("*"),
-    "__div__": Symbol("/"),
-    "__mod__": Symbol("%"),
-    "__eq__": Symbol("=="),
-    "__neq__": Symbol("!="),
-    "__gt__": Symbol(">"),
-    "__ge__": Symbol(">="),
-    "__lt__": Symbol("<"),
-    "__le__": Symbol("<="),
-    "__pow__": Symbol("**"),
-    "__matmul__": Symbol("@"),
-    "__floordiv__": Symbol("//"),
-    "__and__": Symbol("&"),
-    "__or__": Symbol("|"),
-    "__xor__": Symbol("^")
+    "__add__": Symbol._reserved("+"),
+    "__sub__": Symbol._reserved("-"),
+    "__mul__": Symbol._reserved("*"),
+    "__div__": Symbol._reserved("/"),
+    "__mod__": Symbol._reserved("%"),
+    "__eq__": Symbol._reserved("=="),
+    "__neq__": Symbol._reserved("!="),
+    "__gt__": Symbol._reserved(">"),
+    "__ge__": Symbol._reserved(">="),
+    "__lt__": Symbol._reserved("<"),
+    "__le__": Symbol._reserved("<="),
+    "__pow__": Symbol._reserved("**"),
+    "__matmul__": Symbol._reserved("@"),
+    "__floordiv__": Symbol._reserved("//"),
+    "__and__": Symbol._reserved("&"),
+    "__or__": Symbol._reserved("|"),
+    "__xor__": Symbol._reserved("^")
 }
 
 UNOP_MAP = {
-    "__pos__": Symbol("+"),
-    "__neg__": Symbol("-"),
-    "__invert__": Symbol("~"),
+    "__pos__": Symbol._reserved("+"),
+    "__neg__": Symbol._reserved("-"),
+    "__invert__": Symbol._reserved("~"),
 }
 
 BUILTIN_MAP = {
@@ -53,6 +53,7 @@ BUILTIN_MAP = {
     "__float__": Symbol("float"),
     "__int__": Symbol("int"),
     "__format__": Symbol("format"),
+    "__list__": Symbol("list"),
 }
 
 class MacroFlags(NamedTuple):
@@ -129,27 +130,29 @@ class Macro(Expr, MutableSequence[Expr]):
             Expression. If a string is given, it will be parsed to Expr object.
         """        
         if isinstance(expr, str):
-            expr = parse(expr)
-        elif not isinstance(expr, Expr):
+            expr_ = parse(expr)
+        elif isinstance(expr, Expr):
+            expr_ = expr
+        else:
             raise TypeError("Cannot insert objects to Macro except for Expr objects.")
         
-        self.args.insert(key, expr)
+        self.args.insert(key, expr_)
         for callback in self._callbacks:
-            callback(expr)
+            callback(expr_)
     
     @overload
     def __getitem__(self, key: int | str) -> Expr: ...
 
     @overload
-    def __getitem__(self, key: slice) -> Macro[Expr]: ...
+    def __getitem__(self, key: slice) -> Macro: ...
         
     def __getitem__(self, key):
         return self._args[key]
     
-    def __setitem__(self, key: int, value: Expr):
+    def __setitem__(self, key: int, value: Expr) -> None:
         self._args[key] = value
     
-    def __delitem__(self, key: int):
+    def __delitem__(self, key: int) -> None:
         del self._args[key]
     
     def __len__(self) -> int:
@@ -383,7 +386,7 @@ class mObject:
     Abstract class for macro recorder equipped objects.
     """    
     def __init__(self, obj, macro: Macro, returned_callback: MetaCallable = None, 
-                 namespace: Symbol|Expr = None, record_returned: bool = True) -> None:
+                 namespace: Symbol | Expr | None = None, record_returned: bool = True) -> None:
         self.obj = obj
         
         if returned_callback is not None:
@@ -427,14 +430,14 @@ class mObject:
     def macro(self) -> Macro:
         return self._macro
 
-_GET = Symbol.var("__get__", Callable)
-_SET = Symbol.var("__set__", Callable)
-_DELETE = Symbol.var("__delete__", Callable)
+_GET = Symbol.var("__get__")
+_SET = Symbol.var("__set__")
+_DELETE = Symbol.var("__delete__")
 
 class mCallable(mObject):
     obj: Callable
     def __init__(self, function: Callable, macro: Macro, returned_callback: MetaCallable = None,
-                 namespace: Symbol|Expr = None, record_returned: bool = True):
+                 namespace: Symbol | Expr | None = None, record_returned: bool = True):
         super().__init__(function, macro, returned_callback, namespace, record_returned)
         if self.__name__ == "__get__":
             def make_expr(*args, **kwargs):
@@ -473,7 +476,7 @@ class mFunction(mCallable):
     Macro recorder equipped functions. Generated functions are also compatible with methods.
     """
     def __init__(self, function: Callable, macro: Macro, returned_callback: MetaCallable = None,
-                 namespace: Symbol|Expr = None, record_returned: bool = True):
+                 namespace: Symbol | Expr | None = None, record_returned: bool = True):
         super().__init__(function, macro, returned_callback, namespace, record_returned)
         self._method_type = None
             
@@ -596,7 +599,7 @@ class mClassMethod(mCallable): # TODO: inherit classmethod class if possible
     Macro recorder equipped classmethod class.
     """
     def __init__(self, function: classmethod, macro: Macro, returned_callback: MetaCallable = None, 
-                 namespace: Symbol | Expr = None, record_returned: bool = True):
+                 namespace: Symbol | Expr | None = None, record_returned: bool = True):
         super().__init__(function, macro, returned_callback, namespace, record_returned)
         self.__self__ = function.__self__
         clsname = Symbol(self.__self__.__name__)
@@ -725,22 +728,32 @@ class mModule(mObject):
     Macro recorder equipped module class.
     """
     obj: ModuleType
+    def __init__(self, obj, macro: Macro, returned_callback: MetaCallable = None, 
+                 namespace: Symbol | Expr | None = None, record_returned: bool = True) -> None:
+        super().__init__(obj, macro, returned_callback, namespace, record_returned)
+        self.__all__ = getattr(obj, "__all__", [])
+        
     def __getattr__(self, key: str):
         try:
             attr = getattr(self.obj, key)
-            _type = "module" if inspect.ismodule(attr) else "function"
+            is_module = inspect.ismodule(attr)
         except AttributeError:
             try:
                 attr = import_module("." + key, self.obj.__name__)
-                _type = "module"
+                is_module = True
             except ModuleNotFoundError:
                 raise ValueError(f"No function or submodule named '{key}'.")
         
-        if _type == "module":
+        if is_module:
             mmod = mModule(attr, self._macro, self.returned_callback, self.to_namespace(self.obj))
-            setattr(self, key, mmod)
+            setattr(self, key, mmod) # cache
             return mmod
-            
+        
+        elif not isinstance(attr, Callable):
+            # constants, such as "__version__"
+            # TODO: should we record this as getattr?
+            return attr
+        
         @wraps(attr)
         def mfunc(*args, **kwargs):
             with self._macro.blocked():
