@@ -1,11 +1,15 @@
 from __future__ import annotations
-from functools import wraps
 import inspect
-from typing import Callable, Any, TypeVar, overload
+from typing import Callable, Any, TypeVar, overload, TypedDict
 from types import FunctionType, BuiltinFunctionType, ModuleType, MethodType
 
 T = TypeVar("T")
 
+class SymbolDict(TypedDict):
+    name: str
+    object_id: int
+    constant: bool
+
 class Symbol:
     # Map of how to convert object into a symbol.
     _type_map: dict[type[T], Callable[[T], Any]] = {
@@ -13,7 +17,6 @@ class Symbol:
         FunctionType: lambda e: e.__name__,
         BuiltinFunctionType: lambda e: e.__name__,
         MethodType: lambda e: e.__name__,
-        ModuleType: lambda e: e.__name__.split(".")[-1],
         type(None): lambda e: "None",
         str: repr,
         bytes: repr,
@@ -30,13 +33,23 @@ class Symbol:
     # ID of global variables
     _variables: set[int] = set()
     
-    def __init__(self, seq: str, object_id: int = None):
-        self.name = str(seq)
+    # Module symbols
+    _module_symbols: dict[int, Symbol] = {}
+    _module_map: dict[str, ModuleType] = {}
+    
+    def __init__(self, seq: Any, object_id: int = None):
+        self._name = str(seq)
         self.object_id = object_id or id(seq)
         self.constant = True
     
+    def asdict(self) -> SymbolDict:
+        return {"name": self.name, 
+                "object_id": self.object_id, 
+                "constant": self.constant
+                }
+    
     @property
-    def name(self):
+    def name(self) -> str:
         return self._name
     
     @name.setter
@@ -55,16 +68,19 @@ class Symbol:
         return self._name
     
     def __hash__(self) -> int:
-        return self.object_id
+        # To ensure Symbol to be hash safe, we have to consider both object ID and whether
+        # the Symbol is a constant because object ID of a variable is defined by the hash
+        # value of the identifier. 
+        return self.object_id * 2 + int(self.constant)
     
-    def __eq__(self, other: Symbol) -> bool:
+    def __eq__(self, other) -> bool:
         if not isinstance(other, Symbol):
             raise TypeError(f"'==' is not supported between Symbol and {type(other)}")
         return (self.object_id == other.object_id and 
                 self.constant == other.constant)
     
     @classmethod
-    def var(cls, identifier: str, type: type = object):
+    def var(cls, identifier: str):
         """
         Make a variable symbol. Same indentifier with same type always returns identical symbol.
         """        
@@ -72,22 +88,22 @@ class Symbol:
             raise TypeError("'identifier' must be str")
         elif not identifier.isidentifier():
             raise ValueError(f"'{identifier}' is not a valid identifier.")
-        self = cls(identifier, 0)
-        self.object_id = hash(identifier)
+        self = cls(identifier, hash(identifier))
         self.constant = False
         return self
     
-    def as_parameter(self, default=inspect._empty):
+    @classmethod
+    def _reserved(cls, identifier: str):
+        # Users should never use this!!
+        self = cls(identifier, hash(identifier))
+        self.constant = False
+        return self
+    
+    def as_parameter(self, default=inspect._empty) -> inspect.Parameter:
         return inspect.Parameter(self._name, 
                                  inspect.Parameter.POSITIONAL_OR_KEYWORD, 
                                  default=default)
-    
-    def replace(self, other: Symbol) -> None:
-        self.name = other.name
-        self.object_id = other.object_id
-        self.constant = other.constant
-        return None
-    
+        
     @overload
     @classmethod
     def register_type(cls, type: type[T]) -> Callable[[Callable[[T], Any]], Callable[[T], Any]]: ...
@@ -96,6 +112,10 @@ class Symbol:
     @classmethod
     def register_type(cls, function: Callable[[T], Any])-> Callable[[type[T]], type[T]]: ...
     
+    @overload
+    @classmethod
+    def register_type(cls, type: type[T], function: Callable[[T], Any] | None): ...
+    
     @classmethod
     def register_type(cls, type_or_function, function=None):
         """
@@ -135,16 +155,12 @@ class Symbol:
         elif isinstance(type_or_function, Callable):
             if function is not None:
                 raise TypeError("")
-            def _register(type_):
+            def _register(type_: type):
                 cls._type_map[type_] = type_or_function
                 return type_
             return _register
         
         else:
-            raise TypeError()
+            raise TypeError("Arguments of 'register_type' must be type and/or function.")
         
-@wraps(Symbol.register_type)
-def register_type(type_or_function, function = None):
-    return Symbol.register_type(type_or_function, function)
-
-del wraps
\ No newline at end of file
+register_type = Symbol.register_type
